/*-
 * Copyright (c) 1998 John Galbraith
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Sun Jul 26 16:37:58 MDT 1998
 */

/* AT-GPIB register offsets */

#define	DIR 	0x00
#define CDOR	0x00
#define GPIBC   0x01
#define ISR1    0x02
#define IMR1    0x02
#define GPIBD   0x03
#define ISR2    0x04
#define IMR2    0x04
#define KEYPRT  0x05
#define ACCWR   0x05
#define	SPSR	0x06
#define	SPMR	0x06
#define INTR    0x07
#define	ADSR	0x08
#define	ADMR	0x08
#define CNT2    0x09
#define CPTR	0x0a
#define AUXMR	0x0a
#define CNT3    0x0b
#define ADR0	0x0c
#define ADR     0x0c
#define HSSEL   0x0d
#define	ADR1	0x0e
#define	EOSR	0x0e
#define STS1    0x10
#define CFG     0x10
#define DSR     0x11
#define SH_CNT  0x11
#define IMR3    0x12
#define HIER    0x13
#define CNT0    0x14
#define MISC    0x15
#define CNT1    0x16
#define	CSR     0x17
#define	KEYREG	0x17
#define FIFOB   0x18
#define FIFOA   0x19
#define ISR3    0x1a
#define CCR     0x1a
#define SASR	0x1b
#define DCR     0x1b
#define STS2    0x1c
#define CMDR    0x1c
#define ISR0    0x1d
#define IMR0	0x1d
#define TIMER   0x1e
#define	BSR	0x1f
#define	BCR	0x1f

#define GP_ATN  0x80
#define GP_DAV  0x40
#define GP_NDAC 0x20
#define GP_NRFD 0x10
#define GP_EOI  0x08
#define GP_SRQ  0x04
#define GP_IFC  0x02
#define GP_REN  0x01

/* ISR3 masks */
#define ISR3_INT     0x80
#define ISR3_INTSRC2 0x40
#define ISR3_STOP    0x10
#define ISR3_NFF     0x08
#define ISR3_NEF     0x04
#define ISR3_TCLINT  0x02
#define ISR3_DONE    0x01

/* ISR2 masks */
#define ISR2_INT     0x80
#define ISR2_SRQI    0x40
#define ISR2_LOK     0x20
#define ISR2_REM     0x10
#define ISR2_CO      0x08
#define ISR2_LOKC    0x04
#define ISR2_REMC    0x02
#define ISR2_ADSC    0x01

/* ISR1 masks */
#define ISR1_CPT     0x80
#define ISR1_APT     0x40
#define ISR1_DET     0x20
#define ISR1_END     0x10
#define ISR1_DEC     0x08
#define ISR1_ERR     0x04
#define ISR1_DO      0x02
#define ISR1_DI      0x01


/* CMDR commands */
#define CMDR_CLRSC      0x02  /* clear SC (system controller) bit */
#define CMDR_SETSC      0x03  /* set SC bit */
#define CMDR_GO         0x04  /* start DMA */
#define CMDR_STOP       0x08  /* stop DMA  */
#define CMDR_RESET_FIFO 0x10  /* reset FIFO */
#define CMDR_SOFT_RESET 0x22

/* AUXMR commands - nec7210 only  (no special NAT7210 commands) */
#define AUX_PON        0x00  /* immediate execute power on (pon) */
#define AUX_N_IST      0x01  /* clear parallel poll flag (~ist) */
#define AUX_RST        0x02  /* chip reset (ch_rst) */
#define AUX_RHDF       0x03  /* finish handshake (rhdf) */
#define AUX_TRIGGER    0x04  /* trigger (trig) */
#define AUX_RTL_PULSE  0x05  /* clear or pulse return to local (rtl) */
#define AUX_SEOI       0x06  /* send EOI (seoi) */
#define AUX_NONVALID   0x07  /* Nonvalid secondary command or address */
#define AUX_IST        0x09  /* set parallel poll flag (ist) */
#define AUX_RTL_SET    0x0D  /* set return to local */
#define AUX_VALID      0x0F  /* valid secondary command or address */
#define AUX_GTS        0x10  /* go to standby (gts) */
#define AUX_TCA        0x11  /* take control asynchronously (tca) */
#define AUX_TCS        0x12  /* take control ynchronously (tcs) */
#define AUX_LTN        0x13  /* listen (ltn) */
#define AUX_N_RSC      0x14  /* disable system control */
#define AUX_CIFC       0x16  /* Clear IFC (~sic & rsc) */
#define AUX_CREN       0x17  /* Clear REN (~sre & rsc) */
#define AUX_TCSE       0x1A  /* take control syncronously on END (tcse) */
#define AUX_LICM       0x1B  /* listen in Continuous Mode (ltn & cont) */
#define AUX_LUN        0x1C  /* local unlisten (lun) */
#define AUX_RPP        0x1D  /* Execute parallel poll (rpp1) */
#define AUX_SIFC       0x1E  /* set IFC (sic & rsc) */
#define AUX_SREN       0x1F  /* set REN (sre & rsc) */

#ifdef GPIB_DEBUG
char* gpib_commands[] =
{
    "Error",
    "GTL (go to local)",
    "Error",
    "Error",
    "SDC (selected device clear)",
    "PPC (parallel poll configure)",
    "Error",
    "Error",
    "GET (group execute trigger)",
    "TCT (take control)",
    "Error",
    "Error",
    "Error",
    "Error",
    "Error",
    "Error",
    "Error",
    "LLO (local lockout)",
    "Error",
    "Error",
    "DCL (device clear)",
    "PPU (parallel poll unconfigure)",
    "Error",
    "Error",
    "SPE (serial poll enable)",
    "SPD (serial poll disable)",
    "Error",
    "Error",
    "Error",
    "Error",
    "Error",
    "Error",
    "MLA0",
    "MLA1",
    "MLA2",
    "MLA3",
    "MLA4",
    "MLA5",
    "MLA6",
    "MLA7",
    "MLA8",
    "MLA9",
    "MLA10",
    "MLA11",
    "MLA12",
    "MLA13",
    "MLA14",
    "MLA15",
    "MLA16",
    "MLA17",
    "MLA18",
    "MLA19",
    "MLA20",
    "MLA21",
    "MLA22",
    "MLA23",
    "MLA24",
    "MLA25",
    "MLA26",
    "MLA27",
    "MLA28",
    "MLA29",
    "MLA30",
    "UNL (unlisten)",
    "MTA0",
    "MTA1",
    "MTA2",
    "MTA3",
    "MTA4",
    "MTA5",
    "MTA6",
    "MTA7",
    "MTA8",
    "MTA9",
    "MTA10",
    "MTA11",
    "MTA12",
    "MTA13",
    "MTA14",
    "MTA15",
    "MTA16",
    "MTA17",
    "MTA18",
    "MTA19",
    "MTA20",
    "MTA21",
    "MTA22",
    "MTA23",
    "MTA24",
    "MTA25",
    "MTA26",
    "MTA27",
    "MTA28",
    "MTA29",
    "MTA30",
    "UNT, (untalk)",
    "MSA0,PPE",
    "MSA1,PPE",
    "MSA2,PPE",
    "MSA3,PPE",
    "MSA4,PPE",
    "MSA5,PPE",
    "MSA6,PPE",
    "MSA7,PPE",
    "MSA8,PPE",
    "MSA9,PPE",
    "MSA10,PPE",
    "MSA11,PPE",
    "MSA12,PPE",
    "MSA13,PPE",
    "MSA14,PPE",
    "MSA15,PPE",
    "MSA16,PPD",
    "MSA17,PPD",
    "MSA18,PPD",
    "MSA19,PPD",
    "MSA20,PPD",
    "MSA21,PPD",
    "MSA22,PPD",
    "MSA23,PPD",
    "MSA24,PPD",
    "MSA25,PPD",
    "MSA26,PPD",
    "MSA27,PPD",
    "MSA28,PPD",
    "MSA29,PPD",
    "MSA30,PPD",
    "Error"
};

char* gpib_write_registers[] = 
{
    "CDOR",
    "Error",
    "IMR1",
    "Error",
    "IMR2",
    "ACCWR",
    "SPMR",
    "INTR",
    "ADMR",
    "CNT2",
    "AUXMR",
    "CNT3",
    "ADR",
    "HSSEL",
    "EOSR",
    "Error",
    "CFG",
    "SH_CNT",
    "IMR3",
    "HIER",
    "CNT0",
    "MISC",
    "CNT1",
    "KEYREG",
    "FIFOB",
    "FIFOA",
    "CCR",
    "DCR",
    "CMDR",
    "IMR0",
    "TIMER",
    "BCR"    
};

char* gpib_read_registers[] = 
{
    "DIR",
    "Error",
    "ISR1",
    "Error",
    "ISR2",
    "Error",
    "SPSR",
    "Error",
    "ADSR",
    "CNT2",
    "CPTR",
    "CNT3",
    "ADR0",
    "Error",
    "ADR1",
    "Error",
    "STS1",
    "DSR",
    "IMR3",
    "Error",
    "CNT0",
    "Error",
    "CNT1",
    "CSR",
    "FIFOB",
    "FIFOA",
    "ISR3",
    "SASR",
    "STS2",
    "ISR0",
    "TIMER",
    "BSR"    
};

#endif /* GPIB_DEBUG */
