#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <sys/conf.h>
#include <sys/time.h>

#include "../module/gpib.h"

int main(int argc, char* argv[])
{
    int device, gpctl;
    long bufsize = 64*1024;
    char buf[bufsize];
    char answer[bufsize];
    struct gpibconfig info;
    struct timeval start, stop;
    long bytecount, i;

    device = open("/dev/gp0d0", O_RDWR | GPIB_NOCONTROLLER);
    if(device == -1)
    {
        perror("/dev/gp0d0");
        exit(-1);
    }

    gpctl = open("/dev/gp0ctl", O_RDWR | GPIB_NOCONTROLLER);
    if(gpctl == -1)
    {
        perror("/dev/gp0ctl");
        exit(-1);
    }

    if(ioctl(gpctl, GPIB_GET_CONFIG, &info) == -1)
    {
        perror("gp ioctl getconfig");
	exit(-1);
    }
    printf("address: %d, eos: 0x%x, poll_timeout: %d, irq_timeout: %d, eoi: %d\n",
           info.address, info.eos, info.poll_timeout, info.irq_timeout, 
           info.eoi);
    info.poll_timeout = 100000;
    info.irq_timeout = 100000;
    if(ioctl(gpctl, GPIB_SET_CONFIG, &info) == -1)
    {
        perror("gp ioctl setconfig");
	exit(-1);
    }

    for(i = 0; i < bufsize; i++)
        answer[i] = argv[1][0] + i%26;

#if 0
    for(i = 0; i < 4; i++)
    {
        bytecount = read(device, buf, bufsize);
        if(bytecount == -1)
        {
            perror("gp read");
            exit(-1);
        }
        buf[bytecount] = 0;
        printf("read %d bytes: %s\n", bytecount, buf);
    }      
    close(device);
    close(gpctl);
    exit(0);
#endif

    while(1)
    {
        bytecount = 0;
        gettimeofday(&start, 0);
        for(i = 0; i < 50; i++)
        {
            long check, j, count;

            count = read(device, buf, bufsize);
            if(count == -1)
            {
                perror("gp read");
                exit(-1);
            }
            bytecount += count;
            check = 1;
            for(j = 0; j < count; j++)
                if(buf[j] != answer[j])
                    check = 0;
            if(!check)
            {
                printf("error: wrong answers %d (%d bytes)\n", i, count);
/*                 buf[count] = 0; */
/*                 printf("     got: %s\n", buf); */
/*                 answer[count] = 0; */
/*                 printf("expected: %s\n", answer); */
/*                 exit(-1); */
            }
/*             else */
/*                 printf("check %d (%d bytes)\n", i, count); */
        }

        gettimeofday(&stop, 0);
        printf("sent %f bytes per second\n", 1.0*bytecount/(stop.tv_sec-start.tv_sec+stop.tv_usec*1e-6-start.tv_usec*1e-6));
    }
}
